<?php

namespace Drupal\webform_allowed_charactersets\Service;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

/**
 * Characterset service.
 */
class CharactersetService {

  /**
   * Add validation to input fields.
   *
   * Recursively loops a given form or element.
   * Adds element validation handler to each input field.
   *
   * @param array $element
   *   The Form API form or a child element to check if it is or has an input field.
   */
  public function addCharactersetValidation(&$element) {
    // If its an input field.
    if (is_array($element) && isset($element['#type']) && in_array($element['#type'], ['textfield', 'textarea'])) {
      // Add extra validation.
      $element['#element_validate'][] = [$this, 'validateCharacterset'];
    }
    // Recursive loop nested children.
    foreach (Element::children($element) as $key) {
      $child = &$element[$key];
      $this->addCharactersetValidation($child);
    }
  }

  /**
   * Validates the characters in form input fields.
   *
   * - Recursively get all form input fields.
   * - Get each inputted value.
   * - Check for invalid charactersets.
   * - Trigger validation error on the element if invalid characters are found.
   *
   * @param $element
   *   The Form API form array, or a child element.
   * @param FormStateInterface $form_state
   *   The FormState to get the value from and to set the validation error.
   */
  public function validateFormInput($element, FormStateInterface $form_state) {
    // If current element is an input field.
    if (isset($element['#webform_key']) && is_array($element) && isset($element['#type']) && in_array($element['#type'], ['textfield', 'textarea'])) {
      // Get input value.
      $key = $element['#webform_key'];
      $input = $form_state->getValue($key);
      // Validate.
      $validation_messages = $this->isValidCharacterset($input);
      if (!empty($validation_messages)) {
        foreach ($validation_messages as $message) {
          $form_state->setError($element, $message);
        }
      }
    }
    // Recursive loop nested children.
    foreach (Element::children($element) as $key) {
      $child = &$element[$key];
      $this->validateFormInput($child, $form_state);
    }
  }

  /**
   * #element_validate handler for webform input fields.
   *
   * Checks if the given input is valid according with the allowed characterset settings.
   */
  public function validateCharacterset(array $element, FormStateInterface $form_state) {
    if (!isset($element['#webform_key'])) {
      return;
    }
    // Get the input value.
    $key = $element['#webform_key'];
    $input_text = $form_state->getValue($key);
    // Validate.
    $validation_messages = $this->isValidCharacterset($input_text);
    if (!empty($validation_messages)) {
      foreach ($validation_messages as $message) {
        $form_state->setError($element, $message);
      }
    }
  }

  private function isValidCharacterset($text) {

    // Character set regular expressions.
    $characterset_regexes = [
      "latin" => '\p{Latin}',
      "cyrillic" => '\p{Cyrillic}',
      "chinese" => '\x{4e00}-\x{9fff}',
      "japanese" => '\p{Hiragana}\p{Katakana}\p{Han}',
      "korean" => '\x{1100}-\x{11FF}\x{3130}-\x{318F}\x{AC00}-\x{D7AF}',
      "arabic" => '\p{Arabic}',
      "greek" => '\p{Greek}',
      "hebrew" => '\p{Hebrew}',
      "devanagari" => '\p{Devanagari}',
      "thai" => '\p{Thai}',
      "tamil" => '\p{Tamil}',
      "bengali" => '\p{Bengali}',
      "punjabi" => '\p{Gurmukhi}',
      "gujarati" => '\p{Gujarati}',
      "kannada" => '\p{Kannada}',
      "telugu" => '\p{Telugu}',
      "malayalam" => '\p{Malayalam}',
      "georgian" => '\p{Georgian}',
      "armenian" => '\p{Armenian}',
      "tibetan" => '\p{Tibetan}'
      // Add more character sets here
    ];

    $enabled_charactersets = $this->getEnabledCharactersets();
    $validation_messages = [];

    // Loop all possible charactersets so we can always identify the input.
    foreach ($characterset_regexes as $characterset => $regex) {
      // Get the regex for checking that characterset.
      $regex = $characterset_regexes[$characterset];
      // If the text is a valid enabled characterset: return empty array.
      if (preg_match('/[' . $regex . ']/u', $text) && in_array($characterset, $enabled_charactersets)) {
        return [];
      }
      // If the text is not a valid enabled characterset: add message.
      if (preg_match('/[' . $regex . ']/u', $text) && !in_array($characterset, $enabled_charactersets)) {
        $shorttext = $text;
        if (strlen($text) > 15) {
          $shorttext = substr($text, 0, 15) . '...';
        }
        $validation_messages[] = t('Your input contains @characterset characters, which are not allowed: @text', [
          '@characterset' => $characterset,
          '@text' => $shorttext,
        ]);
      }
    }
    return $validation_messages;
  }

  /**
   * Get array of the enabled charactersets
   *
   * @return array
   */
  private function getEnabledCharactersets() {
    $enabled_charactersets = [];
    // Get and loop the valid charactersets.
    $config = \Drupal::config('webform_allowed_charactersets.settings');
    $charactersets = $config->get('charactersets');
    foreach ($charactersets as $characterset => $enabled) {
      if ($enabled) {
        $enabled_charactersets[$characterset] = $characterset;
      }
    }
    return $enabled_charactersets;
  }

}
